@@ -98,11 +98,19 @@
 
 (define *cload-cflags* (if (provided? 'clang) "-fPIC" ""))
 (define *cload-ldflags* "")
-(if (not (defined? '*cload-directory*))
-    (define *cload-directory* ""))
-
-(define *cload-c-compiler* (if (provided? 'gcc) "gcc" (if (provided? 'clang) "clang" "cc")))
+(unless (defined? '*cload-directory*)
+  (define *cload-directory* ""))
+(unless (defined? '*cload-library-name*)
+  (define *cload-library-name* #f))
 
+(define *cload-c-compiler* ;(if (provided? 'tcc)  ; how to get tcc to write a .so file?
+			       ;"tcc"
+			       (if (and (provided? 'gcc) 
+					(not (provided? 'openbsd))) 
+				   "gcc"
+				   (if (provided? 'clang) 
+				       "clang"
+				       "cc")));)
 
 (define-macro (defvar name value) 
   `(if (not (defined? ',name)) 
@@ -123,7 +131,7 @@
 
   (define handlers (list '(integer s7_is_integer s7_integer s7_make_integer s7_int)
 			 '(boolean s7_is_boolean s7_boolean s7_make_boolean bool)
-			 '(real s7_is_real s7_number_to_real s7_make_real s7_double)
+			 '(real s7_is_real s7_number_to_real_with_caller s7_make_real s7_double)
 
 			 ;; '(complex s7_is_complex #f s7_make_complex s7_Complex)
 			 ;; the typedef is around line 6116 in s7.c, but we also need s7_complex which requires the s7_Complex type
@@ -282,7 +290,8 @@
 	     (lambda (header)
 	       (format p "#include <~A>~%" header))
 	     headers))
-	(format p "#include \"s7.h\"~%~%"))
+	(format p "#include \"s7.h\"~%~%")
+	(format p "static s7_pointer c_pointer_string, string_string, character_string, boolean_string, real_string, complex_string, integer_string;~%"))
       
       (define collides?
 	(let ((all-names (hash-table)))
@@ -361,19 +370,29 @@
 				      (hyphen->space nominal-type)
 				      (type->type-symbol nominal-type)  ;(symbol->string nominal-type)
 				      (if (= num-args 1) 0 (+ i 1)))
-			      (begin
+
+			      (let ((function-name (if *cload-library-name*
+						       (string-append "(" *cload-library-name* " '" scheme-name ")")
+						       (if (> (length prefix) 0)
+							   scheme-name
+							   (substring base-name 4)))))
 				(format pp "  if (~A(arg))~%" (checker true-type))
-				(format pp "    ~A_~D = (~A)~A(~Aarg);~%"
+				(format pp "    ~A_~D = (~A)~A(~Aarg~A);~%"
 					base-name i
 					(hyphen->space nominal-type)
 					(s7->C true-type)                               ; s7_number_to_real which requires 
 					(if (memq s7-type '(boolean real))              ;   the extra sc arg
-					    "sc, " ""))
-				(format pp "  else return(s7_wrong_type_arg_error(sc, __func__, ~D, arg, ~S));~%"
+					    "sc, " 
+					    "")
+					(if (memq s7-type '(boolean real))              ;   the extra trailing caller arg
+					    ", __func__" 
+					    ""))
+				(format pp "  else return(s7_wrong_type_error(sc, s7_make_string_wrapper_with_length(sc, ~S, ~D), ~D, arg, ~A_string));~%"
+					function-name
+					(length function-name)
 					(if (= num-args 1) 0 (+ i 1))
-					(if (symbol? s7-type) 
-					    (symbol->string s7-type) 
-					    (error 'bad-arg (format #f "in ~S, ~S is not a symbol~%" name s7-type)))))))
+					s7-type))))
+
 		      (if (< i (- num-args 1))
 			  (format pp "  p = s7_cdr(p);~%")))))
 		
@@ -406,7 +425,8 @@
 		;; add optimizer connection
 		(when (and (eq? return-type 'double)
 			   (< num-args 5)
-			   (sig-every? (lambda (p) (eq? p 'double)) arg-types))
+			   (sig-every? (lambda (p) (eq? p 'double)) arg-types)
+			   (not (defined? (symbol scheme-name) (rootlet)))) ; see below, double-funcs entry not used if already defined
 		  (let ((local-name #f))
 		    (case num-args
 		      ((0)
@@ -432,6 +452,7 @@
 		    (set! double-funcs (cons (list func-name scheme-name local-name) double-funcs))))
 		
 		(when (and (memq return-type '(int size_t))        ; int (f int|double|void)
+			   (not (defined? (symbol scheme-name) (rootlet))) ; see below, int-funcs entry not used if already defined
 			   (or ;(= num-args 0)
 			    (and (= num-args 1)
 				 (memq (car arg-types) '(int size_t double)))
@@ -539,6 +560,14 @@
 	   signatures)
 	  (format p "  }~%~%"))
 
+	(format p "  string_string = s7_make_semipermanent_string(sc, \"a string\");~%")
+	(format p "  c_pointer_string = s7_make_semipermanent_string(sc, \"a c-pointer\");~%")
+	(format p "  character_string = s7_make_semipermanent_string(sc, \"a character\");~%")
+	(format p "  boolean_string = s7_make_semipermanent_string(sc, \"a boolean\");~%")
+	(format p "  real_string = s7_make_semipermanent_string(sc, \"a real\");~%")
+	(format p "  complex_string = s7_make_semipermanent_string(sc, \"a complex number\");~%")
+	(format p "  integer_string = s7_make_semipermanent_string(sc, \"an integer\");~%")
+
 	(format p "  cur_env = s7_curlet(sc);~%") ; changed from s7_outlet(s7_curlet) 20-Aug-17
 	
 	;; send out any special initialization code
@@ -606,7 +635,7 @@
 	 (lambda (f) 
 	   (let ((scheme-name (f 0))
 		 (base-name   (f 1))
-		 (help        (f 2))
+		 (helpf       (f 2))
 		 (num-args    (f 3))
 		 (opt-args    (if (> (length f) 4) (f 4) 0))
 		 (sig         (and (> (length f) 5) (f 5))))
@@ -616,7 +645,7 @@
 		     base-name
 		     num-args
 		     opt-args
-		     help
+		     helpf
 		     (if (pair? sig) (signatures sig) 'NULL))))
 	 functions)
 	
@@ -668,7 +697,7 @@
 			       o-file-name so-file-name *cload-ldflags* ldflags)))
 	      
 	      ((provided? 'openbsd)
-	       (system (format #f "~A -fPIC -ftrampolines -c ~A -o ~A ~A ~A" 
+	       (system (format #f "~A -fPIC -c ~A -o ~A ~A ~A" 
 			       *cload-c-compiler* c-file-name o-file-name *cload-cflags* cflags))
 	       (system (format #f "~A ~A -shared -o ~A ~A ~A" 
 			       *cload-c-compiler* o-file-name so-file-name *cload-ldflags* ldflags)))
@@ -722,14 +751,16 @@
 	  
 	  (lambda (func)
 	    ;; functions
-	    (if (>= (length func) 3)
-		(apply add-one-function func)
-		(case (car func)
-		  ((in-C)       (format pp "~A~%" (cadr func)))
-		  ((C-init)     (set! inits (cons (cadr func) inits)))
-		  ((C-macro)    (apply add-one-macro (cadr func)))
-		  ((C-function) (collides? (caadr func)) (set! functions (cons (check-doc (cadr func)) functions)))
-		  (else         (apply add-one-constant func)))))))
+	    (if (pair? func)
+		(if (>= (length func) 3)
+		    (apply add-one-function func)
+		    (case (car func)
+		      ((in-C)       (format pp "~A~%" (cadr func)))
+		      ((C-init)     (set! inits (cons (cadr func) inits)))
+		      ((C-macro)    (apply add-one-macro (cadr func)))
+		      ((C-function) (collides? (caadr func)) (set! functions (cons (check-doc (cadr func)) functions)))
+		      (else         (apply add-one-constant func))))
+		(error 'wrong-type-arg "~S (func arg to handle-declaration in cload.scm) should be a pair" func)))))
       
       
       ;; c-define-1 (called in c-define macro above)
